from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_list_facet, menu_object, menu_return, menu_setup
)
from mayan.apps.navigation.classes import SourceColumn
from mayan.apps.views.column_widgets import TwoStateWidget

from .classes import SettingNamespace, Setting
from .column_widgets import WidgetSettingValue
from .links import (
    link_setting_namespace_detail, link_setting_namespace_list,
    link_namespace_root_list, link_setting_edit
)
from .widgets import setting_widget


class SmartSettingsApp(MayanAppConfig):
    app_namespace = 'settings'
    app_url = 'settings'
    has_tests = True
    name = 'mayan.apps.smart_settings'
    verbose_name = _('Smart settings')

    def ready(self):
        super().ready()

        SettingNamespace.load_modules()

        SourceColumn(
            func=lambda context: len(context['object'].settings),
            label=_('Setting count'), include_label=True,
            source=SettingNamespace
        )
        SourceColumn(
            func=lambda context: setting_widget(context['object']),
            label=_('Name'), is_identifier=True, source=Setting
        )
        SourceColumn(
            attribute='serialized_value', include_label=True,
            label=_('Value'), widget=WidgetSettingValue, source=Setting
        )
        SourceColumn(
            attribute='is_overridden', include_label=True, source=Setting,
            widget=TwoStateWidget
        )

        menu_list_facet.bind_links(
            links=(link_setting_namespace_detail,),
            sources=(SettingNamespace,)
        )
        menu_object.bind_links(
            links=(link_setting_edit,), sources=(Setting,)
        )
        menu_return.bind_links(
            links=(link_namespace_root_list,), sources=(
                SettingNamespace, Setting,
                'settings:setting_namespace_list'
            )
        )
        menu_setup.bind_links(
            links=(link_setting_namespace_list,)
        )

        Setting.save_last_known_good()



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0009_auto_20170807_0612'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='workflowinstancelogentry',
            options={
                'ordering': ('datetime',),
                'verbose_name': 'Workflow instance log entry',
                'verbose_name_plural': 'Workflow instance log entries'
            },
        ),
    ]



from django.db.models import Q

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin

from ...models.workflow_models import Workflow, WorkflowRuntimeProxy
from ...tasks import (
    task_launch_all_workflows, task_launch_all_workflow_for,
    task_launch_workflow, task_launch_workflow_for
)

from ..literals import (
    TEST_WORKFLOW_TEMPLATE_INTERNAL_NAME, TEST_WORKFLOW_TEMPLATE_LABEL,
    TEST_WORKFLOW_TEMPLATE_LABEL_EDITED
)


class WorkflowTemplateTestMixin(DocumentTestMixin):
    auto_add_test_workflow_template_test_document_type = False
    auto_create_test_workflow_template = False

    def setUp(self):
        super().setUp()
        self._test_workflow_runtime_proxies = []
        self._test_workflow_template_state_runtime_proxies = []
        self._test_workflow_templates = []

        if self.auto_create_test_workflow_template:
            self._create_test_workflow_template(
                add_test_document_type=self.auto_add_test_workflow_template_test_document_type
            )

    def _create_test_workflow_template(
        self, add_test_document_type=False, auto_launch=True
    ):
        total_test_workflow_templates = len(self._test_workflow_templates)
        label = '{}_{}'.format(
            TEST_WORKFLOW_TEMPLATE_LABEL, total_test_workflow_templates
        )
        internal_name = '{}_{}'.format(
            TEST_WORKFLOW_TEMPLATE_INTERNAL_NAME,
            total_test_workflow_templates
        )

        self._test_workflow_template = Workflow.objects.create(
            auto_launch=auto_launch, label=label,
            internal_name=internal_name
        )
        self._test_workflow_templates.append(
            self._test_workflow_template
        )
        self._test_workflow_runtime_proxy = WorkflowRuntimeProxy.objects.get(
            pk=self._test_workflow_template.pk
        )
        self._test_workflow_runtime_proxies.append(
            self._test_workflow_runtime_proxy
        )

        if add_test_document_type:
            self._test_workflow_template.document_types.add(
                self._test_document_type
            )


class DocumentTypeAddRemoveWorkflowTemplateViewTestMixin(
    WorkflowTemplateTestMixin
):
    def _request_test_document_type_workflow_template_add_remove_get_view(self):
        return self.get(
            viewname='document_states:document_type_workflow_templates',
            kwargs={
                'document_type_id': self._test_document_type.pk
            }
        )

    def _request_test_document_type_workflow_template_add_view(self):
        return self.post(
            viewname='document_states:document_type_workflow_templates',
            kwargs={
                'document_type_id': self._test_document_type.pk
            }, data={
                'available-selection': self._test_workflow_template.pk,
                'available-submit': 'true'
            }
        )

    def _request_test_document_type_workflow_template_remove_view(self):
        return self.post(
            viewname='document_states:document_type_workflow_templates',
            kwargs={
                'document_type_id': self._test_document_type.pk
            }, data={
                'added-selection': self._test_workflow_template.pk,
                'added-submit': 'true'
            }
        )


class WorkflowTemplateTaskTestMixin(WorkflowTemplateTestMixin):
    def _execute_task_launch_all_workflows(self):
        task_launch_all_workflows.apply_async().get()

    def _execute_task_launch_all_workflow_for(self):
        task_launch_all_workflow_for.apply_async(
            kwargs={
                'document_id': self._test_document.pk
            }
        ).get()

    def _execute_task_launch_workflow(self):
        task_launch_workflow.apply_async(
            kwargs={
                'workflow_id': self._test_workflow_template.pk
            }
        ).get()

    def _execute_task_launch_workflow_for(self):
        task_launch_workflow_for.apply_async(
            kwargs={
                'document_id': self._test_document.pk,
                'workflow_id': self._test_workflow_template.pk
            }
        ).get()


class WorkflowTemplateDocumentTypeViewTestMixin(WorkflowTemplateTestMixin):
    def _request_test_workflow_template_document_type_add_remove_get_view(self):
        return self.get(
            viewname='document_states:workflow_template_document_types',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )

    def _request_test_workflow_template_document_type_add_view(self):
        return self.post(
            viewname='document_states:workflow_template_document_types',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }, data={
                'available-selection': self._test_document_type.pk,
                'available-submit': 'true'
            }
        )

    def _request_test_workflow_template_document_type_remove_view(self):
        return self.post(
            viewname='document_states:workflow_template_document_types',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }, data={
                'added-selection': self._test_document_type.pk,
                'added-submit': 'true'
            }
        )


class WorkflowTemplateAPIViewTestMixin(WorkflowTemplateTestMixin):
    def _request_test_workflow_template_create_api_view(
        self, extra_data=None
    ):
        data = {
            'internal_name': TEST_WORKFLOW_TEMPLATE_INTERNAL_NAME,
            'label': TEST_WORKFLOW_TEMPLATE_LABEL,
        }

        if extra_data:
            data.update(extra_data)

        pk_list = list(
            Workflow.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='rest_api:workflow-template-list', data=data
        )

        try:
            self._test_workflow_template = Workflow.objects.get(
                ~Q(pk__in=pk_list)
            )
        except Workflow.DoesNotExist:
            self._test_workflow_template = None

        return response

    def _request_test_workflow_template_delete_api_view(self):
        return self.delete(
            viewname='rest_api:workflow-template-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )

    def _request_test_workflow_template_detail_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )

    def _request_test_workflow_template_edit_via_patch_view(self):
        return self.patch(
            viewname='rest_api:workflow-template-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }, data={'label': TEST_WORKFLOW_TEMPLATE_LABEL_EDITED}
        )

    def _request_test_workflow_template_edit_via_put_view(self):
        return self.put(
            viewname='rest_api:workflow-template-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }, data={
                'internal_name': TEST_WORKFLOW_TEMPLATE_INTERNAL_NAME,
                'label': TEST_WORKFLOW_TEMPLATE_LABEL_EDITED
            }
        )

    def _request_test_workflow_template_image_view_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-image', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )

    def _request_test_workflow_template_list_api_view(self):
        return self.get(viewname='rest_api:workflow-template-list')


class WorkflowTemplateDocumentTypeAPIViewMixin(WorkflowTemplateTestMixin):
    def _request_test_workflow_template_document_type_add_api_view(self):
        return self.post(
            viewname='rest_api:workflow-template-document-type-add',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
            }, data={
                'document_type_id': self._test_document_type.pk
            }
        )

    def _request_test_workflow_template_document_type_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-document-type-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )

    def _request_test_workflow_template_document_type_remove_api_view(self):
        return self.post(
            viewname='rest_api:workflow-template-document-type-remove',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
            }, data={
                'document_type_id': self._test_document_type.pk
            }
        )


class WorkflowToolViewTestMixin(WorkflowTemplateTestMixin):
    def _request_workflow_launch_view(self):
        return self.post(
            viewname='document_states:tool_launch_workflows'
        )


class WorkflowTemplateViewTestMixin(WorkflowTemplateTestMixin):
    def _request_test_workflow_template_create_view(self):
        data = {
            'internal_name': TEST_WORKFLOW_TEMPLATE_INTERNAL_NAME,
            'label': TEST_WORKFLOW_TEMPLATE_LABEL,
        }

        pk_list = list(
            Workflow.objects.values_list('pk', flat=True)
        )

        response = self.post(
            viewname='document_states:workflow_template_create', data=data
        )

        try:
            self._test_workflow_template = Workflow.objects.get(
                ~Q(pk__in=pk_list)
            )
        except Workflow.DoesNotExist:
            self._test_workflow_template = None

        return response

    def _request_test_workflow_template_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_single_delete',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )

    def _request_test_workflow_template_edit_view(self):
        return self.post(
            viewname='document_states:workflow_template_edit', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }, data={
                'auto_launch': True,
                'internal_name': self._test_workflow_template.internal_name,
                'label': TEST_WORKFLOW_TEMPLATE_LABEL_EDITED
            }
        )

    def _request_test_workflow_template_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_list',
        )

    def _request_test_workflow_template_preview_view(self):
        return self.get(
            viewname='document_states:workflow_template_preview', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )



from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.search import search_model_document
from mayan.apps.dynamic_search.tests.mixins.base import SearchTestMixin

from ..permissions import permission_signature_capture_view
from ..search import search_model_signature_capture

from .mixins import SignatureCaptureTestMixin


class DocumentSignatureCaptureSearchTestCase(
    SignatureCaptureTestMixin, SearchTestMixin, GenericDocumentViewTestCase
):
    _test_search_model = search_model_document
    auto_create_test_document_stub = True
    auto_create_test_signature_capture = True
    auto_upload_test_document = False

    def test_search_by_text_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__text': self._test_signature_capture.text
            }
        )
        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_text_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__text': self._test_signature_capture.text
            }
        )

        self.assertTrue(self._test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_text_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__text': self._test_signature_capture.text
            }
        )

        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_first_name_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__first_name': self._test_signature_capture.user.first_name
            }
        )
        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_first_name_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__first_name': self._test_signature_capture.user.first_name
            }
        )

        self.assertTrue(self._test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_user_first_name_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__first_name': self._test_signature_capture.user.first_name
            }
        )

        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_last_name_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__last_name': self._test_signature_capture.user.last_name
            }
        )
        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_last_name_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__last_name': self._test_signature_capture.user.last_name
            }
        )

        self.assertTrue(self._test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_user_last_name_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__last_name': self._test_signature_capture.user.last_name
            }
        )

        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_username_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__username': self._test_signature_capture.user.username
            }
        )
        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_username_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__username': self._test_signature_capture.user.username
            }
        )

        self.assertTrue(self._test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_user_username_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'signature_captures__user__username': self._test_signature_capture.user.username
            }
        )

        self.assertTrue(self._test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class SignatureCaptureSearchTestCase(
    SignatureCaptureTestMixin, SearchTestMixin, GenericDocumentViewTestCase
):
    _test_search_model = search_model_signature_capture
    auto_create_test_document_stub = True
    auto_upload_test_document = False
    auto_create_test_signature_capture = True

    def test_search_by_document_label_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__label': self._test_signature_capture.document.label
            }
        )
        self.assertTrue(self._test_signature_capture not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_document_label_with_access(self):
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__label': self._test_signature_capture.document.label
            }
        )

        self.assertTrue(self._test_signature_capture in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_document_type_label_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__document_type__label': self._test_signature_capture.document.document_type.label
            }
        )
        self.assertTrue(self._test_signature_capture not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_document_type_label_with_access(self):
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'document__document_type__label': self._test_signature_capture.document.document_type.label
            }
        )

        self.assertTrue(self._test_signature_capture in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_text_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'text': self._test_signature_capture.text}
        )
        self.assertTrue(self._test_signature_capture not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_text_with_access(self):
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'text': self._test_signature_capture.text}
        )

        self.assertTrue(self._test_signature_capture in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_username_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'user__username': self._test_case_user.username}
        )

        self.assertTrue(self._test_signature_capture not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_username_with_access(self):
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'user__username': self._test_case_user.username}
        )

        self.assertTrue(self._test_signature_capture in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_first_name_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'user__first_name': self._test_case_user.first_name}
        )

        self.assertTrue(self._test_signature_capture not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_first_name_with_access(self):
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'user__first_name': self._test_case_user.first_name}
        )

        self.assertTrue(self._test_signature_capture in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_last_name_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'user__last_name': self._test_case_user.last_name}
        )

        self.assertTrue(self._test_signature_capture not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_search_by_user_last_name_with_access(self):
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'user__last_name': self._test_case_user.last_name}
        )

        self.assertTrue(self._test_signature_capture in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.sources.classes import DocumentCreateWizardStep
from mayan.apps.sources.wizard_steps import DocumentCreateWizardStepDocumentType
from mayan.apps.views.http import URL

from .forms import CabinetListForm
from .models import Cabinet
from .permissions import permission_cabinet_add_document


class DocumentCreateWizardStepCabinets(DocumentCreateWizardStep):
    form_class = CabinetListForm
    label = _('Select cabinets')
    name = 'cabinet_selection'
    number = 3

    @staticmethod
    def get_cabinets_queryset_for_document(document, user=None):
        queryset_documents = Document.valid.all()

        if user:
            queryset_documents = AccessControlList.objects.restrict_queryset(
                permission=permission_cabinet_add_document,
                queryset=queryset_documents, user=user
            )

        try:
            queryset_documents.get(pk=document.pk)
        except Document.DoesNotExist:
            queryset_cabinets = Cabinet.objects.none()
        else:
            queryset_cabinets = Cabinet.objects.all()

            if user:
                queryset_cabinets = AccessControlList.objects.restrict_queryset(
                    permission=permission_cabinet_add_document, queryset=queryset_cabinets,
                    user=user
                )

        return queryset_cabinets

    @staticmethod
    def get_cabinets_queryset_for_document_type(document_type, user):
        queryset_document_types = DocumentType.objects.all()

        queryset_document_types_restricted = AccessControlList.objects.restrict_queryset(
            permission=permission_cabinet_add_document,
            queryset=queryset_document_types, user=user
        )

        try:
            queryset_document_types_restricted.get(pk=document_type.pk)
        except DocumentType.DoesNotExist:
            queryset_cabinets = Cabinet.objects.none()
        else:
            queryset_cabinets = Cabinet.objects.all()

        return queryset_cabinets

    @classmethod
    def condition(cls, wizard):
        return Cabinet.objects.exists()

    @classmethod
    def get_form_kwargs(self, wizard):
        step_data = wizard.get_cleaned_data_for_step(
            step=DocumentCreateWizardStepDocumentType.name
        )
        user = wizard.request.user

        kwargs = {
            'help_text': _('Cabinets to which the document will be added.'),
            'permission': permission_cabinet_add_document,
            'queryset': Cabinet.objects.none(),
            'user': user
        }

        if step_data:
            document_type = step_data['document_type']

            queryset_cabinets = DocumentCreateWizardStepCabinets.get_cabinets_queryset_for_document_type(
                document_type=document_type, user=user
            )

            kwargs['queryset'] = queryset_cabinets

        return kwargs

    @classmethod
    def done(cls, wizard):
        result = {}
        cleaned_data = wizard.get_cleaned_data_for_step(step=cls.name)
        if cleaned_data:
            result['cabinets'] = [
                str(cabinet.pk) for cabinet in cleaned_data['cabinets']
            ]

        return result

    @classmethod
    def step_post_upload_process(
        cls, document, query_string, source_id, user_id
    ):
        User = get_user_model()

        try:
            user = User.objects.get(pk=user_id)
        except User.DoesNotExist:
            user = None

        queryset_cabinets = DocumentCreateWizardStepCabinets.get_cabinets_queryset_for_document(
            document=document, user=user
        )

        cabinet_id_list = URL(
            query_string=query_string
        ).args.getlist('cabinets')

        for cabinet in queryset_cabinets.filter(pk__in=cabinet_id_list):
            if user:
                cabinet.document_add(document=document, user=user)
            else:
                cabinet._document_add(document=document)


DocumentCreateWizardStep.register(step=DocumentCreateWizardStepCabinets)



from mayan.apps.sources.tests.source_backend_actions import (
    SourceBackendActionDocumentUploadBasic
)

from ..source_backend_actions.mixins import SourceBackendActionMixinCompressedInteractive


class SourceBackendActionDocumentUploadBasicCompressed(
    SourceBackendActionMixinCompressedInteractive,
    SourceBackendActionDocumentUploadBasic
):
    """
    Minimal action for the test source.
    """



from ..events import (
    event_document_created, event_document_file_created,
    event_document_file_edited, event_document_type_changed,
    event_document_version_created, event_document_version_page_created
)
from ..models.document_models import Document
from ..models.document_type_models import DocumentType
from ..permissions import permission_document_change_type

from .base import GenericDocumentTestCase
from .literals import (
    TEST_DOCUMENT_SMALL_CHECKSUM, TEST_FILE_SMALL_FILENAME,
    TEST_DOCUMENT_SMALL_MIMETYPE, TEST_DOCUMENT_SMALL_SIZE
)


class DocumentChangeTypeTestCase(GenericDocumentTestCase):
    def setUp(self):
        super().setUp()
        self._create_test_document_type()

    def test_document_change_type_no_permission(self):
        test_document_type = self._test_document.document_type

        self._clear_events()

        with self.assertRaises(expected_exception=Document.DoesNotExist):
            self._test_document.document_type_change(
                document_type=self._test_document_type,
                user=self._test_case_user
            )

        self.assertEqual(
            self._test_document.document_type, test_document_type
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_change_type_with_document_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_change_type
        )

        test_document_type = self._test_document.document_type

        self._clear_events()

        with self.assertRaises(expected_exception=DocumentType.DoesNotExist):
            self._test_document.document_type_change(
                document_type=self._test_document_type,
                user=self._test_case_user
            )

        self.assertEqual(
            self._test_document.document_type, test_document_type
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_change_type_with_document_type_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_change_type
        )

        test_document_type = self._test_document.document_type

        self._clear_events()

        with self.assertRaises(expected_exception=Document.DoesNotExist):
            self._test_document.document_type_change(
                document_type=self._test_document_type,
                user=self._test_case_user
            )

        self.assertEqual(
            self._test_document.document_type, test_document_type
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_change_type_with_full_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_change_type
        )
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_change_type
        )

        test_document_type = self._test_document.document_type

        self._clear_events()

        self._test_document.document_type_change(
            document_type=self._test_document_type,
            user=self._test_case_user
        )

        self.assertNotEqual(
            self._test_document.document_type, test_document_type
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_type_changed.id)

    def test_trashed_document_change_type_with_full_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_change_type
        )
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_change_type
        )

        test_document_type = self._test_document.document_type

        self._test_document.delete()

        self._clear_events()

        with self.assertRaises(expected_exception=Document.DoesNotExist):
            self._test_document.document_type_change(
                document_type=self._test_document_type,
                user=self._test_case_user
            )

        self.assertEqual(
            self._test_document.document_type, test_document_type
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentTestCase(GenericDocumentTestCase):
    auto_upload_test_document = False

    def test_document_creation(self):
        self._clear_events()

        self._upload_test_document()

        self.assertEqual(
            self._test_document.document_type.label,
            self._test_document_type.label
        )
        self.assertEqual(
            self._test_document.label, TEST_FILE_SMALL_FILENAME
        )

        self.assertEqual(
            self._test_document_file.exists(), True
        )
        self.assertEqual(
            self._test_document_file.size, TEST_DOCUMENT_SMALL_SIZE
        )

        self.assertEqual(
            self._test_document_file.mimetype, TEST_DOCUMENT_SMALL_MIMETYPE
        )
        self.assertEqual(
            self._test_document_file.filename, TEST_FILE_SMALL_FILENAME
        )
        self.assertEqual(self._test_document_file.encoding, 'binary')
        self.assertEqual(
            self._test_document_file.checksum, TEST_DOCUMENT_SMALL_CHECKSUM
        )
        self.assertEqual(
            self._test_document_file.pages.count(), 1
        )

        self.assertEqual(
            self._test_document_version.pages.count(), 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        # Document created

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, self._test_document)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        # Document file created

        self.assertEqual(events[1].action_object, self._test_document)
        self.assertEqual(events[1].actor, self._test_document_file)
        self.assertEqual(events[1].target, self._test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        # Document file edited (MIME type, page count update)

        self.assertEqual(events[2].action_object, self._test_document)
        self.assertEqual(events[2].actor, self._test_document_file)
        self.assertEqual(events[2].target, self._test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        # Document version created

        self.assertEqual(events[3].action_object, self._test_document)
        self.assertEqual(events[3].actor, self._test_document_version)
        self.assertEqual(events[3].target, self._test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        # Document version page created

        self.assertEqual(
            events[4].action_object, self._test_document_version
        )
        self.assertEqual(
            events[4].actor, self._test_document_version_page
        )
        self.assertEqual(
            events[4].target, self._test_document_version_page
        )
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_method_get_absolute_url(self):
        self._create_test_document_stub()

        self._clear_events()

        self.assertTrue(
            self._test_document.get_absolute_url()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.db import migrations

SOURCE_BACKEND_MAPPING = {
    'mayan.apps.sources.source_backends.sane_scanner_backends.SourceBackendSANEScanner': 'mayan.apps.source_sane_scanners.source_backends.SourceBackendSANEScanner',
}


def code_source_backend_path_update(apps, schema_editor):
    Source = apps.get_model(app_label='sources', model_name='Source')

    for key, value in SOURCE_BACKEND_MAPPING.items():
        queryset = Source.objects.using(alias=schema_editor.connection.alias).filter(backend_path=key)
        queryset.update(backend_path=value)


def reverse_code_source_backend_path_update(apps, schema_editor):
    Source = apps.get_model(app_label='sources', model_name='Source')

    for key, value in SOURCE_BACKEND_MAPPING.items():
        queryset = Source.objects.using(alias=schema_editor.connection.alias).filter(backend_path=value)
        queryset.update(backend_path=key)


class Migration(migrations.Migration):
    dependencies = [
        ('sources', '0029_update_source_backend_paths')
    ]

    operations = [
        migrations.RunPython(
            code=code_source_backend_path_update,
            reverse_code=reverse_code_source_backend_path_update
        )
    ]



from rest_framework import status

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import (
    event_ocr_document_version_finished,
    event_ocr_document_version_submitted
)
from ..permissions import permission_document_version_ocr

from .mixins import DocumentOCRAPIViewTestMixin


class DocumentOCRAPIViewTestCase(
    DocumentTestMixin, DocumentOCRAPIViewTestMixin, BaseAPITestCase
):
    def test_document_ocr_submit_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_ocr_submit_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertFalse(
            hasattr(self._test_document.pages.first(), 'ocr_content')
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_ocr_submit_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._clear_events()

        response = self._request_test_document_ocr_submit_api_view()
        self.assertEqual(response.status_code, status.HTTP_202_ACCEPTED)

        self.assertTrue(
            hasattr(self._test_document.pages.first(), 'ocr_content')
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_ocr_document_version_submitted.id
        )

        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].action_object, self._test_document)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(
            events[1].verb, event_ocr_document_version_finished.id
        )

    def test_trashed_document_ocr_submit_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_ocr
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_ocr_submit_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from rest_framework import status

from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..models.favorite_document_models import FavoriteDocument
from ..permissions import permission_document_view

from .mixins.document_mixins import DocumentTestMixin
from .mixins.favorite_document_mixins import (
    FavoriteDocumentAPIViewTestMixin, FavoriteDocumentTestMixin
)


class FavoriteDocumentAPIViewTestCase(
    FavoriteDocumentAPIViewTestMixin, FavoriteDocumentTestMixin,
    DocumentTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False
    auto_create_test_document_stub = True

    def test_favorite_document_create_api_view_no_permission(self):
        favorite_document_count = FavoriteDocument.valid.count()

        self._clear_events()

        response = self._request_test_favorite_document_create_api_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count
        )
        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_favorite_document_create_api_view_with_access(self):
        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_favorite_document_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count + 1
        )
        self.assertEqual(
            self._test_favorite_document.document.pk, self._test_document.pk
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_favorite_document_create_api_view_with_access(self):
        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_favorite_document_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_favorite_document_delete_api_view_no_permission(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()

        self._clear_events()

        response = self._request_test_favorite_document_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_favorite_document_delete_api_view_with_access(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_favorite_document_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_favorite_document_delete_api_view_with_access(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_favorite_document_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            FavoriteDocument.valid.count(), favorite_document_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_favorite_document_detail_api_view_no_permission(self):
        self._test_document_favorite_add()

        self._clear_events()

        response = self._request_test_favorite_document_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_favorite_document_detail_api_view_with_access(self):
        self._test_document_favorite_add()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_favorite_document_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['document']['label'],
            self._test_document.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_favorite_document_detail_api_view_with_access(self):
        self._test_document_favorite_add()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_favorite_document_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_favorite_document_list_api_view_no_permission(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()

        self._clear_events()

        response = self._request_test_favorite_document_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], favorite_document_count - 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_favorite_document_list_api_view_with_access(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_favorite_document_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], favorite_document_count)
        self.assertEqual(
            response.data['results'][0]['document']['label'],
            self._test_document.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_favorite_document_list_api_view_with_access(self):
        self._test_document_favorite_add()

        favorite_document_count = FavoriteDocument.valid.count()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_favorite_document_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], favorite_document_count - 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from mayan.apps.appearance.classes import Icon

icon_document_download_multiple = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_download_single = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_file_download_quick = Icon(
    driver_name='fontawesome', symbol='download'
)



from django.contrib.contenttypes.models import ContentType

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..permissions import permission_events_view

from .mixins import EventTypeTestMixin, EventViewTestMixin


class EventsViewTestCase(
    EventTypeTestMixin, EventViewTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_event_type()
        self._create_test_user()
        self._test_object = self._test_document_type

        content_type = ContentType.objects.get_for_model(
            model=self._test_object
        )

        self.view_arguments = {
            'app_label': content_type.app_label,
            'model_name': content_type.model,
            'object_id': self._test_object.pk
        }

    def test_event_list_view_no_permission(self):
        self._create_test_event(target=self._test_object)

        response = self._request_test_event_list_view()

        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_event_type.label
        )

    def test_event_list_view_with_access(self):
        self._create_test_event(target=self._test_object)

        self.grant_access(
            obj=self._test_object, permission=permission_events_view
        )

        response = self._request_test_event_list_view()

        self.assertContains(
            response=response, status_code=200,
            text=self._test_event_type.label
        )

    def test_object_event_list_view_no_permission(self):
        self._create_test_event(
            actor=self._test_user, action_object=self._test_object
        )

        response = self._request_test_object_event_list_view()
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_event_type.label
        )

    def test_object_event_list_view_with_access(self):
        self._create_test_event(
            actor=self._test_user, action_object=self._test_object
        )

        self.grant_access(
            obj=self._test_object, permission=permission_events_view
        )

        response = self._request_test_object_event_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self._test_event_type.label
        )

    def test_verb_event_list_view_no_permission(self):
        self._create_test_event(
            actor=self._test_user, action_object=self._test_object
        )

        response = self._request_test_verb_event_list_view()
        self.assertContains(
            count=2, response=response, status_code=200,
            text=self._test_event_type.label
        )

    def test_verb_event_list_view_with_access(self):
        self._create_test_event(
            actor=self._test_user, action_object=self._test_object
        )

        self.grant_access(
            obj=self._test_object, permission=permission_events_view
        )

        response = self._request_test_verb_event_list_view()
        self.assertContains(
            count=3, response=response, status_code=200,
            text=self._test_event_type.label
        )


class CurrentUserEventsViewTestCase(
    EventTypeTestMixin, EventViewTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_event_type()
        self._create_test_user()
        self._test_object = self._test_case_user

        content_type = ContentType.objects.get_for_model(
            model=self._test_object
        )

        self.view_arguments = {
            'app_label': content_type.app_label,
            'model_name': content_type.model,
            'object_id': self._test_object.pk
        }

    def test_current_user_events_view_no_permission(self):
        self._create_test_event(
            actor=self._test_case_user, action_object=self._test_object
        )

        response = self._request_test_object_event_list_view()
        self.assertNotContains(
            response=response, status_code=200,
            text=self._test_event_type.label
        )

    def test_current_user_events_view_with_access(self):
        self._create_test_event(
            actor=self._test_case_user, action_object=self._test_object
        )

        self.grant_access(
            obj=self._test_case_user, permission=permission_events_view
        )

        response = self._request_test_object_event_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self._test_event_type.label
        )



import logging

from django.conf import settings
from django.contrib import messages
from django.template import RequestContext
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.converter.layers import layer_saved_transformations
from mayan.apps.converter.permissions import (
    permission_transformation_delete, permission_transformation_edit
)
from mayan.apps.converter.transformations import TransformationResize
from mayan.apps.sources.links import link_document_file_upload
from mayan.apps.views.generics import (
    FormView, MultipleObjectConfirmActionView, MultipleObjectDeleteView,
    SingleObjectDetailView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..events import event_document_viewed
from ..forms.document_file_forms import (
    DocumentFileForm, DocumentFilePreviewForm, DocumentFilePropertiesForm
)
from ..forms.misc_forms import PageNumberForm
from ..icons import (
    icon_document_file_delete, icon_document_file_edit,
    icon_document_file_introspect, icon_document_file_list,
    icon_document_file_preview, icon_document_file_properties_detail,
    icon_document_file_print, icon_document_file_transformation_list_clear,
    icon_document_file_transformation_list_clone
)
from ..literals import DEFAULT_DOCUMENT_FILE_ACTION_NAME
from ..models.document_models import Document
from ..models.document_file_models import DocumentFile
from ..permissions import (
    permission_document_file_delete, permission_document_file_edit,
    permission_document_file_print, permission_document_file_tools,
    permission_document_file_view
)
from ..settings import setting_preview_height, setting_preview_width
from ..tasks import task_document_file_delete, task_document_file_size_update

from .misc_views import PrintFormView, DocumentPrintBaseView

logger = logging.getLogger(name=__name__)


class DocumentFileDeleteView(MultipleObjectDeleteView):
    error_message = _(
        'Error deleting document file "%(instance)s"; %(exception)s'
    )
    object_permission = permission_document_file_delete
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    success_message_single = _(
        'Document file "%(object)s" deletion queued successfully.'
    )
    success_message_singular = _(
        '%(count)d document file deletion queued successfully.'
    )
    success_message_plural = _(
        '%(count)d document files deletions queued successfully.'
    )
    title_single = _('Delete document file: %(object)s.')
    title_singular = _('Delete the %(count)d selected document file.')
    title_plural = _('Delete the %(count)d selected document files.')
    view_icon = icon_document_file_delete

    def get_extra_context(self):
        context = {
            'message': _(
                'All document files pages from this document file and the '
                'document version pages linked to them will be deleted too. '
                'The process will be performed in the background.'
            )
        }

        if self.object_list.count() > 1:
            context.update(
                {
                    'object': self.object_list.first().document
                }
            )

        return context

    def get_post_action_redirect(self):
        return reverse(
            viewname='documents:document_file_list', kwargs={
                'document_id': self.object_list.first().document.pk
            }
        )

    def object_action(self, instance, form=None):
        task_document_file_delete.apply_async(
            kwargs={
                'document_file_id': instance.pk,
                'user_id': self.request.user.pk
            }
        )


class DocumentFileEditView(SingleObjectEditView):
    form_class = DocumentFileForm
    object_permission = permission_document_file_edit
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    view_icon = icon_document_file_edit

    def get_extra_context(self):
        return {
            'title': _('Edit document file: %s') % self.object
        }

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='documents:document_file_preview', kwargs={
                'document_file_id': self.object.pk
            }
        )


class DocumentFileIntrospectView(MultipleObjectConfirmActionView):
    object_permission = permission_document_file_tools
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    success_message = _(
        '%(count)d document file queued for introspection.'
    )
    success_message_plural = _(
        '%(count)d document files queued for introspection.'
    )
    view_icon = icon_document_file_introspect

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Introspect the selected document file?',
                plural='Introspect the selected document files?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        'Introspect the document file: %s?'
                    ) % queryset.first()
                }
            )

        result['message'] = _(
            'The document file will be re-examined for file size, page '
            'count, checksum, and its related document version pages '
            're-created. All transformations will be lost.'
        )

        return result

    def object_action(self, form, instance):
        task_document_file_size_update.apply_async(
            kwargs={
                'action_name': DEFAULT_DOCUMENT_FILE_ACTION_NAME,
                'document_file_id': instance.pk,
                'is_document_upload_sequence': True,
                'user_id': self.request.user.pk
            }
        )


class DocumentFileListView(ExternalObjectViewMixin, SingleObjectListView):
    external_object_permission = permission_document_file_view
    external_object_pk_url_kwarg = 'document_id'
    external_object_queryset = Document.valid.all()
    view_icon = icon_document_file_list

    @staticmethod
    def get_no_results_context(document=None, request=None):
        context = {
            'no_results_icon': icon_document_file_list,
            'no_results_text': _(
                'File are the actual files that were uploaded for each '
                'document. Their contents needs to be mapped to a version '
                'before it can be used.'
            ),
            'no_results_title': _('No files available')
        }

        if document:
            context['no_results_main_link'] = link_document_file_upload.resolve(
                context=RequestContext(
                    dict_={'object': document},
                    request=request
                )
            )

        return context

    def get_document(self):
        document = self.external_object
        document.add_as_recent_document_for_user(user=self.request.user)
        return document

    def get_extra_context(self):
        document = self.get_document()
        context = {
            'hide_object': True,
            'list_as_items': True,
            'object': document,
            'table_cell_container_classes': 'td-container-thumbnail',
            'title': _('Files of document: %s') % document
        }
        context.update(
            DocumentFileListView.get_no_results_context(
                document=document, request=self.request
            )
        )
        return context

    def get_source_queryset(self):
        return self.get_document().files.order_by('-timestamp')


class DocumentFilePreviewView(SingleObjectDetailView):
    form_class = DocumentFilePreviewForm
    object_permission = permission_document_file_view
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    view_icon = icon_document_file_preview

    def dispatch(self, request, *args, **kwargs):
        result = super().dispatch(request, *args, **kwargs)
        self.object.document.add_as_recent_document_for_user(
            user=request.user
        )
        event_document_viewed.commit(
            actor=request.user, action_object=self.object,
            target=self.object.document
        )

        return result

    def get_extra_context(self):
        return {
            'hide_labels': True,
            'object': self.object,
            'title': _('Preview of document file: %s') % self.object
        }

    def get_form_extra_kwargs(self):
        transformation_instance_list = (
            TransformationResize(
                height=setting_preview_height.value,
                width=setting_preview_width.value
            ),
        )

        return {
            'transformation_instance_list': transformation_instance_list
        }


class DocumentFilePrintFormView(PrintFormView):
    external_object_permission = permission_document_file_print
    external_object_pk_url_kwarg = 'document_file_id'
    external_object_queryset = DocumentFile.valid.all()
    print_view_name = 'documents:document_file_print_view'
    print_view_kwarg = 'document_file_id'
    view_icon = icon_document_file_print

    def _add_recent_document(self):
        self.external_object.document.add_as_recent_document_for_user(
            user=self.request.user
        )


class DocumentFilePrintView(DocumentPrintBaseView):
    external_object_permission = permission_document_file_print
    external_object_pk_url_kwarg = 'document_file_id'
    external_object_queryset = DocumentFile.valid.all()
    view_icon = icon_document_file_print

    def _add_recent_document(self):
        self.external_object.document.add_as_recent_document_for_user(
            user=self.request.user
        )


class DocumentFilePropertiesView(SingleObjectDetailView):
    form_class = DocumentFilePropertiesForm
    object_permission = permission_document_file_view
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    view_icon = icon_document_file_properties_detail

    def dispatch(self, request, *args, **kwargs):
        result = super().dispatch(request, *args, **kwargs)
        self.object.document.add_as_recent_document_for_user(
            user=request.user
        )
        return result

    def get_extra_context(self):
        return {
            'document_file': self.object,
            'object': self.object,
            'title': _('Properties of document file: %s') % self.object
        }


class DocumentFileTransformationsClearView(MultipleObjectConfirmActionView):
    object_permission = permission_transformation_delete
    pk_url_kwarg = 'document_file_id'
    source_queryset = DocumentFile.valid.all()
    success_message = _(
        'Transformation clear request processed for %(count)d document file.'
    )
    success_message_plural = _(
        'Transformation clear request processed for %(count)d document files.'
    )
    view_icon = icon_document_file_transformation_list_clear

    def get_extra_context(self):
        result = {
            'title': ungettext(
                singular='Clear all the page transformations for the selected document file?',
                plural='Clear all the page transformations for the selected document files?',
                number=self.object_list.count()
            )
        }

        if self.object_list.count() == 1:
            result.update(
                {
                    'object': self.object_list.first(),
                    'title': _(
                        'Clear all the page transformations for the '
                        'document file: %s?'
                    ) % self.object_list.first()
                }
            )

        return result

    def object_action(self, form, instance):
        try:
            for page in instance.pages.all():
                layer_saved_transformations.get_transformations_for(
                    obj=page
                ).delete()
        except Exception as exception:
            messages.error(
                message=_(
                    'Error deleting the page transformations for '
                    'document file: %(document_file)s; %(error)s.'
                ) % {
                    'document_file': instance, 'error': exception
                }, request=self.request
            )


class DocumentFileTransformationsCloneView(ExternalObjectViewMixin, FormView):
    external_object_permission = permission_transformation_edit
    external_object_pk_url_kwarg = 'document_file_id'
    external_object_queryset = DocumentFile.valid.all()
    form_class = PageNumberForm
    view_icon = icon_document_file_transformation_list_clone

    def dispatch(self, request, *args, **kwargs):
        results = super().dispatch(request=request, *args, **kwargs)
        self.external_object.document.add_as_recent_document_for_user(
            user=request.user
        )

        return results

    def form_valid(self, form):
        try:
            layer_saved_transformations.copy_transformations(
                delete_existing=True, source=form.cleaned_data['page'],
                targets=form.cleaned_data['page'].siblings.exclude(
                    pk=form.cleaned_data['page'].pk
                )
            )
        except Exception as exception:
            if settings.DEBUG:
                raise
            else:
                messages.error(
                    message=_(
                        'Error cloning the page transformations for '
                        'document file: %(document_file)s; %(error)s.'
                    ) % {
                        'document_file': self.external_object,
                        'error': exception
                    }, request=self.request
                )
        else:
            messages.success(
                message=_('Transformations cloned successfully.'),
                request=self.request
            )

        return super().form_valid(form=form)

    def get_form_extra_kwargs(self):
        return {
            'instance': self.external_object
        }

    def get_extra_context(self):
        context = {
            'object': self.external_object,
            'title': _(
                'Clone page transformations of document file: %s'
            ) % self.external_object
        }

        return context



from ..interfaces import (
    SourceBackendActionInterface, SourceBackendActionInterfaceRequestRESTAPI,
    SourceBackendActionInterfaceRequestViewForm,
    SourceBackendActionInterfaceTask
)

from .arguments import argument_document_description


class SourceBackendActionMixinDocumentDescriptionInteractive:
    class Interface:
        class Model(SourceBackendActionInterface):
            class Argument:
                description = argument_document_description

            def process_interface_context(self):
                super().process_interface_context()

                self.action_kwargs['description'] = self.context['description']

        class RESTAPI(SourceBackendActionInterfaceRequestRESTAPI):
            class Argument:
                description = argument_document_description

            def process_interface_context(self):
                super().process_interface_context()

                self.action_kwargs['description'] = self.context['description']

        class Task(SourceBackendActionInterfaceTask):
            class Argument:
                description = argument_document_description

            def process_interface_context(self):
                super().process_interface_context()

                self.action_kwargs['description'] = self.context['description']

        class View(SourceBackendActionInterfaceRequestViewForm):
            def process_interface_context(self):
                super().process_interface_context()

                form_document = self.context['forms']['document_form']
                form_document_cleaned_data = form_document.cleaned_data

                self.action_kwargs['description'] = form_document_cleaned_data.get(
                    'description', None
                )

    def get_document_task_kwargs(self, description=None, **kwargs):
        result = super().get_document_task_kwargs(**kwargs)

        result['description'] = description

        return result

    def get_task_kwargs(self, description=None, **kwargs):
        result = super().get_task_kwargs(**kwargs)

        result['action_interface_kwargs']['description'] = description

        return result



import logging
import json

import requests

from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.credentials.class_mixins import BackendMixinCredentialsOptional

from .classes import WorkflowAction
from .exceptions import WorkflowStateActionError
from .literals import (
    BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT,
    DEFAULT_HTTP_ACTION_TIMEOUT
)
from .models.workflow_instance_models import WorkflowInstance
from .models.workflow_models import Workflow
from .permissions import permission_workflow_tools
from .tasks import task_launch_workflow_for

logger = logging.getLogger(name=__name__)


class DocumentPropertiesEditAction(WorkflowAction):
    form_fields = {
        'document_label': {
            'label': _('Document label'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {}',
                        _('The new label to be assigned to the document.'),
                        BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT
                    )
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': False
            },
        }, 'document_description': {
            'label': _('Document description'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {}',
                        _(
                            'The new description to be assigned to the '
                            'document.'
                        ), BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT
                    )
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': False
            }
        }
    }
    label = _('Modify document properties')

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Document properties'), {
                    'fields': ('document_label', 'document_description')
                },
            ),
        )
        return fieldsets

    def execute(self, context):
        self.document_label = self.kwargs.get('document_label')
        self.document_description = self.kwargs.get(
            'document_description'
        )
        new_label = None
        new_description = None

        if self.document_label:
            new_label = self.render_field(
                field_name='document_label', context=context
            )

        if self.document_description:
            new_description = self.render_field(
                field_name='document_description', context=context
            )

        if new_label or new_description:
            document = context['workflow_instance'].document
            document.label = new_label or document.label
            document.description = new_description or document.description

            document.save()


class DocumentWorkflowLaunchAction(WorkflowAction):
    form_field_widgets = {
        'workflows': {
            'class': 'django.forms.widgets.SelectMultiple', 'kwargs': {
                'attrs': {'class': 'select2'},
            }
        }
    }
    label = _('Launch workflows')

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        workflow_template = cls.workflow_template_state.workflow

        workflows_union = Workflow.objects.filter(
            document_types__in=workflow_template.document_types.all()
        ).exclude(pk=workflow_template.pk).distinct()

        fields.update(
            {
                'workflows': {
                    'class': 'mayan.apps.views.fields.ModelFormFieldFilteredModelMultipleChoice',
                    'help_text': _(
                        'Additional workflows to launch for the document.'
                    ),
                    'kwargs': {
                        'source_queryset': workflows_union,
                        'permission': permission_workflow_tools
                    },
                    'label': _('Workflows'),
                    'required': True
                }
            }
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Workflows'), {
                    'fields': ('workflows',)
                },
            ),
        )
        return fieldsets

    def execute(self, context):
        workflows = Workflow.objects.filter(
            pk__in=self.kwargs.get(
                'workflows', ()
            )
        )

        document = context['workflow_instance'].document

        for workflow in workflows:
            task_launch_workflow_for.apply_async(
                kwargs={
                    'document_id': document.pk,
                    'workflow_id': workflow.pk
                }
            )


class HTTPAction(BackendMixinCredentialsOptional, WorkflowAction):
    form_fields = {
        'url': {
            'label': _('URL'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {}',
                        _('The URL to access.'),
                        BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT
                    )
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': True
            },
        }, 'method': {
            'label': _('Method'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {}',
                        _(
                            'The HTTP method to use for the request. '
                            'Typical choices are OPTIONS, HEAD, POST, GET, '
                            'PUT, PATCH, DELETE.'
                        ), BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT
                    )
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': True
            }
        }, 'username': {
            'label': _('Username'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {} {}',
                        _(
                            'Username to use for making the request with '
                            'HTTP Basic Auth.'
                        ), BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT,
                        _(
                            'The credential object is available as '
                            '{{ credential }}.'
                        )
                    )
                ),
                'max_length': 192,
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': False
            },
        }, 'password': {
            'label': _('Password'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {} {}',
                        _(
                            'Password to use for making the request with '
                            'HTTP Basic Auth.'
                        ), BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT,
                        _(
                            'The credential object is available as '
                            '{{ credential }}.'
                        )
                    )
                ),
                'max_length': 255,
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': False
            },
        }, 'headers': {
            'label': _('Headers'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {} {}',
                        _(
                            'Headers to send with the HTTP request. Must '
                            'be in JSON format.'
                        ), BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT,
                        _(
                            'The credential object is available as '
                            '{{ credential }}.'
                        )
                    )
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': False
            }
        }, 'payload': {
            'label': _('Payload'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {}',
                        _('A JSON document to include in the request.'),
                        BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT
                    )
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': False
            }
        }, 'timeout': {
            'label': _('Timeout'),
            'class': 'mayan.apps.templating.fields.ModelTemplateField',
            'default': DEFAULT_HTTP_ACTION_TIMEOUT,
            'kwargs': {
                'initial_help_text': _(
                    format_lazy(
                        '{} {}',
                        _('Time in seconds to wait for a response.'),
                        BASE_WORKFLOW_TEMPLATE_STATE_ACTION_HELP_TEXT
                    )
                ),
                'model': WorkflowInstance,
                'model_variable': 'workflow_instance',
                'required': True
            }
        }

    }
    label = _('Perform an HTTP request')
    previous_dotted_paths = (
        'mayan.apps.document_states.workflow_actions.HTTPPostAction',
    )

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('HTTP'), {
                    'fields': (
                        'url', 'username', 'password', 'headers', 'timeout',
                        'method', 'payload'
                    )
                },
            ),
        )
        return fieldsets

    def render_field_load(self, field_name, context):
        """
        Method to perform a template render and subsequent JSON load.
        """
        render_result = self.render_field(
            field_name=field_name, context=context
        ) or '{}'

        try:
            load_result = json.loads(s=render_result, strict=False)
        except Exception as exception:
            raise WorkflowStateActionError(
                _('%(field_name)s JSON error: %(exception)s') % {
                    'field_name': field_name, 'exception': exception
                }
            )

        logger.debug('load result: %s', load_result)

        return load_result

    def execute(self, context):
        authentication_context = context.copy()

        credential = self.get_credential()
        if credential:
            authentication_context['credential'] = credential

        headers = self.render_field_load(
            field_name='headers', context=authentication_context
        )
        method = self.render_field(field_name='method', context=context)
        password = self.render_field(
            field_name='password', context=authentication_context
        )
        payload = self.render_field_load(
            field_name='payload', context=context
        )
        timeout = self.render_field(field_name='timeout', context=context)
        url = self.render_field(field_name='url', context=context)
        username = self.render_field(
            field_name='username', context=authentication_context
        )

        if '.' in timeout:
            timeout = float(timeout)
        elif timeout:
            timeout = int(timeout)
        else:
            timeout = None

        authentication = None
        if username or password:
            authentication = requests.auth.HTTPBasicAuth(
                password=password, username=username
            )

        requests.request(
            auth=authentication, headers=headers, json=payload,
            method=method, timeout=timeout, url=url
        )



COMMAND_NAME_AUTHENTICATION_OTP_DISABLE = 'authentication_otp_disable'
COMMAND_NAME_AUTHENTICATION_OTP_INITIALIZE = 'authentication_otp_initialize'
COMMAND_NAME_AUTHENTICATION_OTP_STATUS = 'authentication_otp_status'



from django.conf.urls import url

from .api_views import (
    APIDocumentResolvedSmartLinkDetailView,
    APIDocumentResolvedSmartLinkDocumentListView,
    APIDocumentResolvedSmartLinkListView, APISmartLinkListView, APISmartLinkDetailView,
    APISmartLinkConditionListView, APISmartLinkConditionView,
    APISmartLinkDocumentTypeAddView, APISmartLinkDocumentTypeListView,
    APISmartLinkDocumentTypeRemoveView
)
from .views.smart_link_condition_views import (
    SmartLinkConditionCreateView, SmartLinkConditionDeleteView,
    SmartLinkConditionEditView, SmartLinkConditionListView
)
from .views.smart_link_views import (
    DocumentResolvedSmartLinkListView, DocumentTypeSmartLinkAddRemoveView,
    DocumentResolvedSmartLinkDocumentListView, SmartLinkCreateView,
    SmartLinkDeleteView, SmartLinkDocumentTypeAddRemoveView,
    SmartLinkEditView, SmartLinkListView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/smart_links/$',
        name='document_smart_link_instance_list',
        view=DocumentResolvedSmartLinkListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/smart_links/(?P<smart_link_id>\d+)/$',
        name='smart_link_instance_view',
        view=DocumentResolvedSmartLinkDocumentListView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/smart_links/$',
        name='document_type_smart_links',
        view=DocumentTypeSmartLinkAddRemoveView.as_view()
    ),
    url(
        regex=r'^smart_links/$', name='smart_link_list',
        view=SmartLinkListView.as_view()
    ),
    url(
        regex=r'^smart_links/create/$', name='smart_link_create',
        view=SmartLinkCreateView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>\d+)/delete/$',
        name='smart_link_delete', view=SmartLinkDeleteView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>\d+)/document_types/$',
        name='smart_link_document_types',
        view=SmartLinkDocumentTypeAddRemoveView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>\d+)/edit/$',
        name='smart_link_edit', view=SmartLinkEditView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>\d+)/conditions/$',
        name='smart_link_condition_list',
        view=SmartLinkConditionListView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>\d+)/conditions/create/$',
        name='smart_link_condition_create',
        view=SmartLinkConditionCreateView.as_view()
    ),
    url(
        regex=r'^smart_links/conditions/(?P<smart_link_condition_id>\d+)/delete/$',
        name='smart_link_condition_delete',
        view=SmartLinkConditionDeleteView.as_view()
    ),
    url(
        regex=r'^smart_links/conditions/(?P<smart_link_condition_id>\d+)/edit/$',
        name='smart_link_condition_edit',
        view=SmartLinkConditionEditView.as_view()
    )
]

api_urls = [
    url(
        regex=r'^smart_links/$', name='smartlink-list',
        view=APISmartLinkListView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>[0-9]+)/$', name='smartlink-detail',
        view=APISmartLinkDetailView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>[0-9]+)/conditions/$',
        name='smartlinkcondition-list',
        view=APISmartLinkConditionListView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>[0-9]+)/conditions/(?P<smart_link_condition_id>[0-9]+)/$',
        name='smartlinkcondition-detail',
        view=APISmartLinkConditionView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>[0-9]+)/document_types/$',
        name='smartlink-document_type-list',
        view=APISmartLinkDocumentTypeListView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>[0-9]+)/document_types/add/$',
        name='smartlink-document_type-add',
        view=APISmartLinkDocumentTypeAddView.as_view()
    ),
    url(
        regex=r'^smart_links/(?P<smart_link_id>[0-9]+)/document_types/remove/$',
        name='smartlink-document_type-remove',
        view=APISmartLinkDocumentTypeRemoveView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/resolved_smart_links/$',
        name='resolvedsmartlink-list',
        view=APIDocumentResolvedSmartLinkListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/resolved_smart_links/(?P<resolved_smart_link_id>[0-9]+)/$',
        name='resolvedsmartlink-detail',
        view=APIDocumentResolvedSmartLinkDetailView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/resolved_smart_links/(?P<resolved_smart_link_id>[0-9]+)/documents/$',
        name='resolvedsmartlinkdocument-list',
        view=APIDocumentResolvedSmartLinkDocumentListView.as_view()
    )
]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig


class SourceEmailsApp(MayanAppConfig):
    app_namespace = 'source_emails'
    app_url = 'source_emails'
    has_rest_api = False
    has_static_media = False
    has_tests = True
    name = 'mayan.apps.source_emails'
    verbose_name = _('Emails')



from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..events import event_workflow_instance_created
from ..permissions import permission_workflow_tools

from .mixins.workflow_tool_mixins import WorkflowToolViewTestMixin


class WorkflowToolViewTestCase(
    WorkflowToolViewTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()
        self._create_test_workflow_template(add_test_document_type=True)
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()

    def test_tool_launch_workflows_view_no_permission(self):
        workflow_instance_count = self._test_document.workflows.count()

        self._clear_events()

        response = self._request_workflow_launch_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(
            self._test_document.workflows.count(), workflow_instance_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_tool_launch_workflows_view_with_permission(self):
        self.grant_permission(permission=permission_workflow_tools)

        workflow_instance_count = self._test_document.workflows.count()

        self._clear_events()

        response = self._request_workflow_launch_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_document.workflows.count(),
            workflow_instance_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].target, self._test_document.workflows.first()
        )
        self.assertEqual(events[0].verb, event_workflow_instance_created.id)

    def test_trashed_document_tool_launch_workflows_view_with_permission(self):
        self.grant_permission(permission=permission_workflow_tools)

        workflow_instance_count = self._test_document.workflows.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_workflow_launch_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_document.workflows.count(), workflow_instance_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.core import management

from mayan.apps.cabinets.models import Cabinet

from ..base import MountCommandMixin

DEFAULT_DELIMITER = ','


class Command(MountCommandMixin, management.BaseCommand):
    help = 'Mount a cabinet as a filesystem.'
    node_identifier_argument = 'label'
    node_identifier_help_text = 'Cabinet label'
    node_text_attribute = 'label'

    def add_extra_arguments(self, parser):
        parser.add_argument(
            '--delimiter', action='store', dest='delimiter',
            default=DEFAULT_DELIMITER, help='Character used to separate '
            'the cabinet levels. Defaults to "{}".'.format(DEFAULT_DELIMITER)
        )

    def factory_func_document_container_node(self, *args, **options):
        levels = options[self.node_identifier_argument].split(options['delimiter'])

        cabinet = None
        for level in levels:
            try:
                if cabinet is None:
                    cabinet = Cabinet.objects.get(label=level)
                else:
                    cabinet = Cabinet.objects.get(label=level, parent=cabinet)
            except Cabinet.DoesNotExist as exception:
                self.stderr.write(
                    msg='Cabinet level "{}" not found; {}'.format(
                        level, exception
                    )
                )
                exit(1)

        def func_document_container_node():
            return cabinet

        return func_document_container_node



from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.document_indexing.models.index_instance_models import IndexInstanceNode
from mayan.apps.document_indexing.tests.mixins import IndexTemplateTestMixin

from .literals import TEST_TAG_LABEL_EDITED, TEST_TAG_INDEX_NODE_TEMPLATE
from .mixins import TagTestMixin


class TagIndexingTestCase(
    IndexTemplateTestMixin, TagTestMixin, GenericDocumentTestCase
):
    _test_index_template_node_expression = TEST_TAG_INDEX_NODE_TEMPLATE
    auto_create_test_document_stub = True
    auto_create_test_tag = True
    auto_upload_test_document = False

    def test_indexing_not_tag(self):
        value = self._test_tag.label

        self.assertFalse(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=value
            ).exists()
        )

    def test_indexing_tag_attach(self):
        self._test_tag.attach_to(
            document=self._test_document, user=self._test_case_user
        )

        value = self._test_tag.label

        self.assertTrue(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=value
            ).exists()
        )

    def test_indexing_tag_delete(self):
        self._test_tag.attach_to(
            document=self._test_document, user=self._test_case_user
        )

        value = self._test_tag.label

        self._test_tag.delete()

        self.assertFalse(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=value
            ).exists()
        )

    def test_indexing_tag_edit(self):
        self._test_tag.attach_to(
            document=self._test_document, user=self._test_case_user
        )
        value = self._test_tag.label

        self._test_tag.label = TEST_TAG_LABEL_EDITED
        self._test_tag.save()
        value_edited = self._test_tag.label

        self.assertFalse(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=value
            ).exists()
        )
        self.assertTrue(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=value_edited
            ).exists()
        )

    def test_indexing_tag_remove(self):
        self._test_tag.attach_to(
            document=self._test_document, user=self._test_case_user
        )
        self._test_tag.remove_from(
            document=self._test_document, user=self._test_case_user
        )
        value = self._test_tag.label

        self.assertFalse(
            IndexInstanceNode.objects.filter(
                documents=self._test_document,
                value=value
            ).exists()
        )



from mayan.apps.documents.tests.base import GenericDocumentTestCase

from .mixins.workflow_instance_mixins import WorkflowInstanceTestMixin
from .mixins.workflow_template_transition_field_mixins import WorkflowTemplateTransitionFieldTestMixin


class WorkflowTemplateTransitionFieldModelTestCase(
    WorkflowInstanceTestMixin, WorkflowTemplateTransitionFieldTestMixin,
    GenericDocumentTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_workflow_template(add_test_document_type=True)
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()
        self._create_test_workflow_template_transition_field()
        self._create_test_document_stub()

    def test_deleted_field_context_references(self):
        """
        Transition a workflow with a transition field, and then delete the
        transition field. The retrieving the context should work even with an
        obsolete field reference.
        """
        self._do_test_workflow_instance_transition(
            extra_data={
                self._test_workflow_template_transition_field.name: 'test'
            }
        )
        self._test_document.workflows.first().log_entries.first().get_extra_data()
        self._test_workflow_template_transition_field.delete()
        self._test_document.workflows.first().log_entries.first().get_extra_data()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

from .literals import (
    DEFAULT_COMMON_COLLAPSE_LIST_MENU_LIST_FACET,
    DEFAULT_COMMON_COLLAPSE_LIST_MENU_OBJECT,
    DEFAULT_COMMON_DISABLE_LOCAL_STORAGE, DEFAULT_COMMON_DISABLED_APPS,
    DEFAULT_COMMON_EXTRA_APPS, DEFAULT_COMMON_EXTRA_APPS_PRE,
    DEFAULT_COMMON_HOME_VIEW, DEFAULT_COMMON_HOME_VIEW_DASHBOARD_NAME,
    DEFAULT_COMMON_PROJECT_TITLE
)

namespace = SettingNamespace(
    label=_('Common'), name='common', version='0002'
)

setting_collapse_list_menu_list_facet = namespace.add_setting(
    choices=('false', 'true'),
    default=DEFAULT_COMMON_COLLAPSE_LIST_MENU_LIST_FACET,
    global_name='COMMON_COLLAPSE_LIST_MENU_LIST_FACET',
    help_text=_(
        'In list mode, show the "list facet" menu options as a dropdown '
        'menu instead of individual buttons.'
    )
)
setting_collapse_list_menu_object = namespace.add_setting(
    choices=('false', 'true'),
    default=DEFAULT_COMMON_COLLAPSE_LIST_MENU_OBJECT,
    global_name='COMMON_COLLAPSE_LIST_MENU_OBJECT',
    help_text=_(
        'In list mode, show the "object" menu options as a dropdown menu '
        'instead of individual buttons.'
    )
)
setting_disable_local_storage = namespace.add_setting(
    choices=('false', 'true'),
    default=DEFAULT_COMMON_DISABLE_LOCAL_STORAGE,
    global_name='COMMON_DISABLE_LOCAL_STORAGE', help_text=_(
        'Disables the use of the local `media` folder. When using this '
        'setting, all apps must be also configured via their respective '
        'storage backend settings to use alternate persistence.'
    )
)
setting_disabled_apps = namespace.add_setting(
    default=DEFAULT_COMMON_DISABLED_APPS,
    global_name='COMMON_DISABLED_APPS', help_text=_(
        'A list of strings designating all applications that are to be '
        'removed from the list normally installed by Mayan EDMS. Each '
        'string should be a dotted Python path to: an application '
        'configuration class (preferred), or a package containing an '
        'application. Example: [\'app_1\', \'app_2\']'
    )
)
setting_extra_apps = namespace.add_setting(
    default=DEFAULT_COMMON_EXTRA_APPS, global_name='COMMON_EXTRA_APPS',
    help_text=_(
        'A list of strings designating all applications that are installed '
        'beyond those normally installed by Mayan EDMS. Each string '
        'should be a dotted Python path to: an application configuration '
        'class (preferred), or a package containing an application. '
        'Example: [\'app_1\', \'app_2\']'
    )
)
setting_extra_apps = namespace.add_setting(
    default=DEFAULT_COMMON_EXTRA_APPS_PRE,
    global_name='COMMON_EXTRA_APPS_PRE',
    help_text=_(
        'A list of strings designating all applications that are installed '
        'beyond those normally installed by Mayan EDMS. Each string '
        'should be a dotted Python path to: an application configuration '
        'class (preferred), or a package containing an application. '
        'These apps will be installed before the default apps. '
        'Example: [\'app_1\', \'app_2\']'
    )
)
setting_home_view = namespace.add_setting(
    default=DEFAULT_COMMON_HOME_VIEW, global_name='COMMON_HOME_VIEW',
    help_text=_(
        'Name of the view attached to the brand anchor in the main menu. '
        'This is also the view to which users will be redirected after '
        'log in.'
    )
)
setting_home_view_dashboard = namespace.add_setting(
    default=DEFAULT_COMMON_HOME_VIEW_DASHBOARD_NAME,
    global_name='COMMON_HOME_VIEW_DASHBOARD_NAME',
    help_text=_(
        'Name of the dashboard to display in the home view.'
    )
)
setting_project_title = namespace.add_setting(
    default=DEFAULT_COMMON_PROJECT_TITLE, global_name='COMMON_PROJECT_TITLE',
    help_text=_('Sets the project\'s name.')
)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('document_parsing', '0010_auto_20201012_0708')
    ]

    operations = [
        migrations.DeleteModel(
            name='DocumentFileParseError',
        )
    ]



# Generated by Django 2.2.15 on 2020-10-21 20:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('checkouts', '0010_auto_20200917_0643'),
    ]

    operations = [
        migrations.DeleteModel(
            name='NewFileBlock',
        ),
    ]



import uuid

from django.db import models, migrations
from django.core.files.storage import FileSystemStorage


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0002_auto_20150608_1902')
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='uuid',
            field=models.CharField(
                default=str(
                    uuid.uuid4()
                ), editable=False, max_length=48
            ),
            preserve_default=True
        ),
        migrations.AlterField(
            model_name='documentversion',
            name='file',
            field=models.FileField(
                storage=FileSystemStorage(), upload_to=str(
                    uuid.uuid4()
                ), verbose_name='File'
            ),
            preserve_default=True
        )
    ]



from django.conf.urls import url

from .api_views import (
    APIDocumentDuplicateListView, APIDuplicatedDocumentListView
)
from .views import (
    DocumentDuplicatesListView, DuplicatedDocumentListView,
    ScanDuplicatedDocuments
)

urlpatterns = [
    url(
        regex=r'^documents/duplicated/$',
        name='duplicated_document_list',
        view=DuplicatedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/duplicates/$',
        name='document_duplicates_list',
        view=DocumentDuplicatesListView.as_view()
    ),
    url(
        regex=r'^documents/duplicated/scan/$',
        name='duplicated_document_scan',
        view=ScanDuplicatedDocuments.as_view()
    )
]

api_urls = [
    url(
        regex=r'^documents/duplicated/$',
        name='duplicateddocument-list',
        view=APIDuplicatedDocumentListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>[0-9]+)/duplicates/$',
        name='documentduplicate-list',
        view=APIDocumentDuplicateListView.as_view()
    )
]



import logging

from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.generics import SingleObjectEditView

from ..icons import icon_document_type_retention_policies
from ..models.document_type_models import DocumentType
from ..permissions import permission_document_type_edit

logger = logging.getLogger(name=__name__)


class DocumentTypeRetentionPoliciesEditView(SingleObjectEditView):
    fields = (
        'trash_time_unit', 'trash_time_period', 'delete_time_unit',
        'delete_time_period', 'document_stub_pruning_enabled',
        'document_stub_expiration_interval'
    )
    fieldsets = (
        (
            _('Trashing'), {
                'fields': ('trash_time_unit', 'trash_time_period'),
            }
        ), (
            _('Deletion'), {
                'fields': ('delete_time_unit', 'delete_time_period')
            }
        ), (
            _('Stub pruning'), {
                'fields': (
                    'document_stub_pruning_enabled',
                    'document_stub_expiration_interval'
                )
            }
        )
    )
    model = DocumentType
    object_permission = permission_document_type_edit
    pk_url_kwarg = 'document_type_id'
    post_action_redirect = reverse_lazy(
        viewname='documents:document_type_list'
    )
    view_icon = icon_document_type_retention_policies

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _(
                'Retention policies for document type: %s'
            ) % self.object
        }

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }



from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.generics import (
    ConfirmView, SimpleView, SingleObjectListView
)

from .classes import StatisticNamespace
from .icons import (
    icon_statistic_detail, icon_statistic_queue,
    icon_statistic_namespace_detail, icon_statistic_namespace_list
)

from .permissions import permission_statistics_view
from .tasks import task_execute_statistic
from .view_mixins import StatisticTypeViewMixin


class StatisticNamespaceListView(SingleObjectListView):
    extra_context = {
        'hide_link': True,
        'no_results_icon': icon_statistic_namespace_list,
        'no_results_text': _(
            'Statistics namespaces group statistics into logical units. '
        ),
        'no_results_title': _('No statistic namespaces available'),
        'title': _('Statistics namespaces')
    }
    template_name = 'appearance/generic_list.html'
    view_icon = icon_statistic_namespace_list
    view_permission = permission_statistics_view

    def get_source_queryset(self):
        return StatisticNamespace.get_all()


class StatisticNamespaceDetailView(SingleObjectListView):
    view_icon = icon_statistic_namespace_detail
    view_permission = permission_statistics_view

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().dispatch(request=request, *args, **kwargs)

    def get_extra_context(self):
        return {
            'hide_link': True,
            'no_results_icon': icon_statistic_namespace_detail,
            'no_results_text': _(
                'Statistics are metrics and chart representations of '
                'existing data.'
            ),
            'no_results_title': _('No statistic available'),
            'object': self.object,
            'title': _('Namespace details for: %s') % self.object
        }

    def get_object(self):
        return StatisticNamespace.get(
            slug=self.kwargs['slug']
        )

    def get_source_queryset(self):
        return self.object.statistics


class StatisticTypeDetailView(StatisticTypeViewMixin, SimpleView):
    view_icon = icon_statistic_detail
    view_permission = permission_statistics_view

    def get_extra_context(self):
        return {
            'chart_context': self.object.get_chart_context(),
            'namespace': self.object.namespace,
            'navigation_object_list': ('namespace', 'object'),
            'no_data': not self.object.get_results_data()['series'],
            'object': self.object,
            'title': _('Results for: %s') % self.object
        }

    def get_template_names(self):
        return (self.object.renderer.template_name,)


class StatisticTypeQueueView(StatisticTypeViewMixin, ConfirmView):
    view_icon = icon_statistic_queue
    view_permission = permission_statistics_view

    def get_extra_context(self):
        return {
            'namespace': self.object.namespace,
            'navigation_object_list': ('namespace', 'object'),
            'object': self.object,
            # Translators: This text is asking users if they want to queue
            # (to send to the queue) a statistic for it to be update ahead
            # of schedule
            'title': _(
                'Queue statistic "%s" to be updated?'
            ) % self.object
        }

    def view_action(self):
        task_execute_statistic.delay(slug=self.object.slug)
        messages.success(
            message=_(
                'Statistic "%s" queued successfully for update.'
            ) % self.object.label, request=self.request
        )



import logging

from django.apps import apps
from django.contrib.auth import get_user_model

from mayan.celery import app

logger = logging.getLogger(name=__name__)


@app.task(ignore_result=True)
def task_launch_all_workflows(user_id=None):
    Document = apps.get_model(app_label='documents', model_name='Document')
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    User = get_user_model()

    if user_id:
        user = User.objects.get(pk=user_id)
    else:
        user = None

    logger.info('Start launching workflows')
    for document in Document.valid.all():
        Workflow.objects.launch_for(document=document, user=user)

    logger.info('Finished launching workflows')


@app.task(ignore_result=True)
def task_launch_workflow(workflow_id, user_id=None):
    Document = apps.get_model(app_label='documents', model_name='Document')
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    User = get_user_model()

    if user_id:
        user = User.objects.get(pk=user_id)
    else:
        user = None

    workflow = Workflow.objects.get(pk=workflow_id)

    logger.info('Start launching workflow: %d', workflow_id)
    documents_queryset = Document.valid.filter(
        document_type__in=workflow.document_types.all()
    )
    for document in documents_queryset:
        workflow.launch_for(document=document, user=user)

    logger.info('Finished launching workflow: %d', workflow_id)


@app.task(ignore_result=True)
def task_launch_workflow_for(document_id, workflow_id, user_id=None):
    Document = apps.get_model(app_label='documents', model_name='Document')
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    User = get_user_model()

    if user_id:
        user = User.objects.get(pk=user_id)
    else:
        user = None

    document = Document.valid.get(pk=document_id)
    workflow = Workflow.objects.get(pk=workflow_id)

    logger.info(
        'Start launching workflow: %d for document: %d',
        workflow_id, document_id
    )
    workflow.launch_for(document=document, user=user)

    logger.info(
        'Finished launching workflow: %d for document: %d', workflow_id,
        document_id
    )


@app.task(ignore_result=True)
def task_launch_all_workflow_for(document_id):
    Document = apps.get_model(app_label='documents', model_name='Document')
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    document = Document.valid.get(pk=document_id)

    logger.info(
        'Start launching all workflows for document: %d', document_id
    )
    Workflow.objects.launch_for(document=document)

    logger.info(
        'Finished launching all workflows for document: %d', document_id
    )


@app.task(ignore_result=True)
def task_workflow_instance_check_escalation(workflow_instance_id):
    WorkflowInstance = apps.get_model(
        app_label='document_states', model_name='WorkflowInstance'
    )

    workflow_instance = WorkflowInstance.objects.get(pk=workflow_instance_id)
    workflow_instance.check_escalation()


@app.task(ignore_result=True)
def task_workflow_instance_check_escalation_all():
    WorkflowInstance = apps.get_model(
        app_label='document_states', model_name='WorkflowInstance'
    )
    WorkflowStateEscalation = apps.get_model(
        app_label='document_states', model_name='WorkflowStateEscalation'
    )

    # Filter workflow instances whose workflow template have at least
    # one state with expiration enabled.
    queryset_workflow_templates = WorkflowStateEscalation.objects.values(
        'state__workflow'
    )

    queryset_workflow_instance = WorkflowInstance.valid.filter(
        workflow__in=queryset_workflow_templates
    )

    for workflow_instance in queryset_workflow_instance:
        task_workflow_instance_check_escalation.apply_async(
            kwargs={'workflow_instance_id': workflow_instance.pk}
        )



from importlib import import_module
import logging
from django.apps import apps

logger = logging.getLogger(name=__name__)


class AppsModuleLoaderMixin:
    # __loader_module_sets is used to avoid double imports, it should not
    # be modified by the user.
    __loader_module_sets = {}

    # _loader_module_name must be set to the module name that is to be
    # uploaded by the class mixin.
    _loader_module_name = None

    @classmethod
    def load_modules(cls):
        # This set keeps track of what apps have already been processed.
        cls.__loader_module_sets.setdefault(
            cls._loader_module_name, set()
        )

        for app in apps.get_app_configs():
            if app not in cls.__loader_module_sets[cls._loader_module_name]:
                try:
                    import_module(
                        name='{}.{}'.format(
                            app.name, cls._loader_module_name
                        )
                    )
                except ImportError as exception:
                    # Determine which errors during import should be ignored
                    # and which are serious enough to raise.
                    non_fatal_messages = (
                        'No module named {module_name}'.format(
                            module_name=cls._loader_module_name
                        ),
                        'No module named \'{app_label}.{module_name}\''.format(
                            app_label=app.name, module_name=cls._loader_module_name
                        )
                    )
                    if str(exception) not in non_fatal_messages:
                        logger.error(
                            'Error importing %s %s.py file; %s', app.name,
                            cls._loader_module_name, exception, exc_info=True
                        )
                        raise
                finally:
                    cls.__loader_module_sets[
                        cls._loader_module_name
                    ].add(app)

        cls.post_load_modules()

    @classmethod
    def post_load_modules(cls):
        return



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

from .literals import (
    DEFAULT_MAXIMUM_FAILED_PRUNE_ATTEMPTS,
    DEFAULT_MAXIMUM_NORMAL_PRUNE_ATTEMPTS
)

namespace = SettingNamespace(
    label=_('File caching'), name='file_caching'
)

setting_maximum_failed_prune_attempts = namespace.add_setting(
    default=DEFAULT_MAXIMUM_FAILED_PRUNE_ATTEMPTS,
    global_name='FILE_CACHING_MAXIMUM_FAILED_PRUNE_ATTEMPTS', help_text=_(
        'Number of times a cache will retry failed attempts to prune '
        'files to free up space for new a file being requested, before '
        'giving up.'
    )
)
setting_maximum_normal_prune_attempts = namespace.add_setting(
    default=DEFAULT_MAXIMUM_NORMAL_PRUNE_ATTEMPTS,
    global_name='FILE_CACHING_MAXIMUM_NORMAL_PRUNE_ATTEMPTS', help_text=_(
        'Number of times a cache will attempt to prune files to free up '
        'space for new a file being requested, before giving up.'
    )
)



import json

from django.db.models import Q

from ...models import Source

from ..literals import (
    TEST_CASE_ACTION_NAME_SOURCE_CREATE, TEST_CASE_INTERFACE_NAME_REST_API,
    TEST_SOURCE_ACTION_CONFIRM_FALSE_NAME,
    TEST_SOURCE_ACTION_CONFIRM_TRUE_NAME, TEST_SOURCE_LABEL_EDITED
)

from .base_mixins import SourceTestMixin


class SourceActionAPIViewTestMixin(SourceTestMixin):
    def _request_test_source_action_detail_api_view(self):
        return self.get(
            viewname='rest_api:source_action-detail', kwargs={
                'action_name': TEST_SOURCE_ACTION_CONFIRM_FALSE_NAME,
                'source_id': self._test_source.pk
            }
        )

    def _request_test_source_action_execute_get_api_view(
        self, action_name=None, extra_query=None
    ):
        action_name = action_name or TEST_SOURCE_ACTION_CONFIRM_FALSE_NAME

        backend_data = self.get_test_source_backend_data(
            action_name=action_name,
            interface_name=TEST_CASE_INTERFACE_NAME_REST_API,
            extra_data=extra_query
        )

        return self.get(
            viewname='rest_api:source_action-execute', kwargs={
                'action_name': action_name,
                'source_id': self._test_source.pk
            }, query=backend_data
        )

    def _request_test_source_action_execute_post_api_view(
        self, action_name=None, extra_arguments=None, test_file_path=None
    ):
        action_name = action_name or TEST_SOURCE_ACTION_CONFIRM_TRUE_NAME

        backend_data = self.get_test_source_backend_data(
            action_name=action_name,
            interface_name=TEST_CASE_INTERFACE_NAME_REST_API,
            extra_data=extra_arguments
        )

        data = {
            'arguments': json.dumps(obj=backend_data)
        }

        if test_file_path:
            self._test_source_file_path = test_file_path

        with self.get_test_source_file_object() as file_object:
            if file_object:
                data['file'] = file_object

            return self.post(
                viewname='rest_api:source_action-execute', kwargs={
                    'action_name': action_name,
                    'source_id': self._test_source.pk
                }, data=data
            )

    def _request_test_source_action_list_api_view(self):
        return self.get(
            viewname='rest_api:source_action-list', kwargs={
                'source_id': self._test_source.pk
            }
        )


class SourceAPIViewTestMixin(SourceTestMixin):
    def _request_test_source_create_api_view(self, extra_data=None):
        pk_list = list(
            Source.objects.values_list('pk', flat=True)
        )

        self._test_source_pre_create()

        backend_data = self.get_test_source_backend_data(
            action_name=TEST_CASE_ACTION_NAME_SOURCE_CREATE,
            interface_name=TEST_CASE_INTERFACE_NAME_REST_API,
            extra_data=extra_data
        )

        label = backend_data.pop('label')

        json_backend_data = json.dumps(obj=backend_data)

        backend_path = self.get_test_source_backend_path()

        response = self.post(
            viewname='rest_api:source-list', data={
                'backend_data': json_backend_data,
                'backend_path': backend_path,
                'label': label
            }
        )
        try:
            self._test_source = Source.objects.get(
                ~Q(pk__in=pk_list)
            )
        except Source.DoesNotExist:
            self._test_source = None

        return response

    def _request_test_source_delete_api_view(self):
        return self.delete(
            viewname='rest_api:source-detail', kwargs={
                'source_id': self._test_source.pk
            }
        )

    def _request_test_source_detail_api_view(self):
        return self.get(
            viewname='rest_api:source-detail', kwargs={
                'source_id': self._test_source.pk
            }
        )

    def _request_test_source_edit_api_view_via_patch(self):
        data = {'label': TEST_SOURCE_LABEL_EDITED}

        return self.patch(
            viewname='rest_api:source-detail', kwargs={
                'source_id': self._test_source.pk
            }, data=data
        )

    def _request_test_source_edit_api_view_via_put(self):
        data = {
            'backend_path': self._test_source.backend_path,
            'enabled': self._test_source.enabled,
            'label': TEST_SOURCE_LABEL_EDITED
        }

        return self.put(
            viewname='rest_api:source-detail', kwargs={
                'source_id': self._test_source.pk
            }, data=data
        )

    def _request_test_source_list_api_view(self):
        return self.get(viewname='rest_api:source-list')



from rest_framework import status

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import (
    event_document_checked_in, event_document_checked_out,
    event_document_forcefully_checked_in
)
from ..permissions import (
    permission_document_check_in, permission_document_check_in_override,
    permission_document_check_out, permission_document_check_out_detail_view
)

from .mixins import (
    CheckoutAPIViewTestMixin, DocumentCheckoutAPIViewTestMixin,
    DocumentCheckoutTestMixin
)


class CheckoutsAPITestCase(
    CheckoutAPIViewTestMixin, DocumentCheckoutTestMixin,
    DocumentTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()

    def test_document_check_out_create_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_check_out_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertFalse(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document, permission=permission_document_check_out
        )

        self._clear_events()

        response = self._request_test_check_out_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_checked_out.id)

    def test_trashed_document_check_out_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document, permission=permission_document_check_out
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_check_out_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertFalse(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_delete_api_view_no_permission(self):
        self._check_out_test_document()

        self._clear_events()

        response = self._request_test_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_delete_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document, permission=permission_document_check_in
        )

        self._clear_events()

        response = self._request_test_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(
            events[0].verb, event_document_checked_in.id
        )

    def test_trashed_document_check_out_delete_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document, permission=permission_document_check_in
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_in_forcefull_api_view_no_permission(self):
        self._create_test_user()
        self._check_out_test_document(user=self._test_user)

        self.grant_access(
            obj=self._test_document, permission=permission_document_check_in
        )

        self._clear_events()

        response = self._request_test_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_in_forcefull_api_view_with_access(self):
        self._create_test_user()
        self._check_out_test_document(user=self._test_user)

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_in_override
        )

        self._clear_events()

        response = self._request_test_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(
            events[0].verb, event_document_forcefully_checked_in.id
        )

    def test_trashed_document_check_in_forcefull_api_view_with_access(self):
        self._create_test_user()
        self._check_out_test_document(user=self._test_user)

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_in_override
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_detail_api_view_no_permission(self):
        self._check_out_test_document()

        self._clear_events()

        response = self._request_test_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_detail_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_out_detail_view
        )

        self._clear_events()

        response = self._request_test_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['document']['uuid'],
            str(self._test_document.uuid)
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_check_out_detail_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_out_detail_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_list_api_view_no_permission(self):
        self._check_out_test_document()

        self._clear_events()

        response = self._request_test_check_out_list_api_view()
        self.assertNotContains(
            response=response, text=self._test_document.uuid,
            status_code=status.HTTP_200_OK
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_list_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_out_detail_view
        )

        self._clear_events()

        response = self._request_test_check_out_list_api_view()
        self.assertContains(
            response=response, text=self._test_document.uuid,
            status_code=status.HTTP_200_OK
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_check_out_list_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_out_detail_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_check_out_list_api_view()
        self.assertNotContains(
            response=response, text=self._test_document.uuid,
            status_code=status.HTTP_200_OK
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentCheckoutAPITestCase(
    DocumentCheckoutAPIViewTestMixin, DocumentCheckoutTestMixin,
    DocumentTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()

    def test_document_check_out_detail_api_view_no_permission(self):
        self._check_out_test_document()

        self._clear_events()

        response = self._request_test_document_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_detail_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_out_detail_view
        )

        self._clear_events()

        response = self._request_test_document_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['document']['uuid'],
            str(self._test_document.uuid)
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_check_out_detail_api_view_with_access(self):
        self._check_out_test_document()

        self._test_document.delete()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_out_detail_view
        )

        self._clear_events()

        response = self._request_test_document_check_out_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_delete_api_view_no_permission(self):
        self._check_out_test_document()

        self._clear_events()

        response = self._request_test_document_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_out_delete_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document, permission=permission_document_check_in
        )

        self._clear_events()

        response = self._request_test_document_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(
            events[0].verb, event_document_checked_in.id
        )

    def test_trashed_document_check_out_delete_api_view_with_access(self):
        self._check_out_test_document()

        self.grant_access(
            obj=self._test_document, permission=permission_document_check_in
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_in_forcefull_api_view_no_permission(self):
        self._create_test_user()
        self._check_out_test_document(user=self._test_user)

        self.grant_access(
            obj=self._test_document, permission=permission_document_check_in
        )

        self._clear_events()

        response = self._request_test_document_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_check_in_forcefull_api_view_with_access(self):
        self._create_test_user()
        self._check_out_test_document(user=self._test_user)

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_in_override
        )

        self._clear_events()

        response = self._request_test_document_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(
            events[0].verb, event_document_forcefully_checked_in.id
        )

    def test_trashed_document_check_in_forcefull_api_view_with_access(self):
        self._create_test_user()
        self._check_out_test_document(user=self._test_user)

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_check_in_override
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_check_out_detail_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self._test_document.is_checked_out())

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.apps import apps


def handler_initialize_new_user_options(sender, instance, **kwargs):
    UserOptions = apps.get_model(
        app_label='user_management', model_name='UserOptions'
    )

    if kwargs['created']:
        UserOptions.objects.create(user=instance)



from mayan.apps.rest_api import generics

from ..models.recently_accessed_document_models import RecentlyAccessedDocument
from ..serializers.recently_accessed_document_serializers import (
    RecentlyAccessedDocumentSerializer
)
from ..permissions import permission_document_view


class APIRecentlyAccessedDocumentListView(generics.ListAPIView):
    """
    get: Return a list of the recently accessed documents for the current user.
    """
    mayan_object_permissions = {
        'GET': (permission_document_view,)
    }
    serializer_class = RecentlyAccessedDocumentSerializer

    def get_source_queryset(self):
        return RecentlyAccessedDocument.valid.filter(user=self.request.user)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0013_document_is_stub'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documenttype',
            options={
                'ordering': ('label',), 'verbose_name': 'Document type',
                'verbose_name_plural': 'Documents types'
            },
        ),
        migrations.RenameField(
            model_name='documenttype',
            old_name='name',
            new_name='label',
        ),
    ]



from mayan.apps.rest_api import generics

from ..permissions import (
    permission_workflow_template_edit, permission_workflow_template_view
)
from ..serializers.workflow_template_state_serializers import (
    WorkflowTemplateStateActionSerializer, WorkflowTemplateStateSerializer
)

from .api_view_mixins import (
    ParentObjectWorkflowTemplateAPIViewMixin,
    ParentObjectWorkflowTemplateStateAPIViewMixin
)


class APIWorkflowTemplateStateListView(
    ParentObjectWorkflowTemplateAPIViewMixin, generics.ListCreateAPIView
):
    """
    get: Returns a list of all the workflow template states.
    post: Create a new workflow template state.
    """
    mayan_external_object_permissions = {
        'GET': (permission_workflow_template_view,),
        'POST': (permission_workflow_template_edit,)
    }
    ordering_fields = ('completion', 'id', 'initial', 'label')
    serializer_class = WorkflowTemplateStateSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user,
            'workflow': self.get_workflow_template()
        }

    def get_source_queryset(self):
        return self.get_workflow_template().states.all()


class APIWorkflowTemplateStateView(
    ParentObjectWorkflowTemplateAPIViewMixin,
    generics.RetrieveUpdateDestroyAPIView
):
    """
    delete: Delete the selected workflow template state.
    get: Return the details of the selected workflow template state.
    patch: Edit the selected workflow template state.
    put: Edit the selected workflow template state.
    """
    mayan_object_permissions = {
        'DELETE': (permission_workflow_template_edit,),
        'GET': (permission_workflow_template_view,),
        'PATCH': (permission_workflow_template_edit,),
        'PUT': (permission_workflow_template_edit,)
    }
    lookup_url_kwarg = 'workflow_template_state_id'
    serializer_class = WorkflowTemplateStateSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }

    def get_source_queryset(self):
        return self.get_workflow_template().states.all()


class APIWorkflowTemplateStateActionListView(
    ParentObjectWorkflowTemplateStateAPIViewMixin, generics.ListCreateAPIView
):
    """
    get: Returns a list of all the workflow template state actions.
    post: Create a new workflow template state action.
    """
    mayan_external_object_permissions = {
        'GET': (permission_workflow_template_view,),
        'POST': (permission_workflow_template_edit,)
    }
    ordering_fields = ('label', 'enabled', 'id')
    serializer_class = WorkflowTemplateStateActionSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user,
            'state': self.get_workflow_template_state()
        }

    def get_source_queryset(self):
        return self.get_workflow_template_state().actions.all()


class APIWorkflowTemplateStateActionDetailView(
    ParentObjectWorkflowTemplateStateAPIViewMixin,
    generics.RetrieveUpdateDestroyAPIView
):
    """
    delete: Delete the selected workflow template state action.
    get: Return the details of the selected workflow template state action.
    patch: Edit the selected workflow template state action.
    put: Edit the selected workflow template state action.
    """
    mayan_object_permissions = {
        'DELETE': (permission_workflow_template_edit,),
        'GET': (permission_workflow_template_view,),
        'PATCH': (permission_workflow_template_edit,),
        'PUT': (permission_workflow_template_edit,)
    }
    lookup_url_kwarg = 'workflow_template_state_action_id'
    serializer_class = WorkflowTemplateStateActionSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }

    def get_source_queryset(self):
        return self.get_workflow_template_state().actions.all()



from mayan.apps.testing.tests.base import GenericViewTestCase

from ..events import event_error_log_deleted
from ..permissions import (
    permission_error_log_entry_delete, permission_error_log_entry_view
)

from .mixins import (
    ErrorLogPartitionEntryTestMixin, ErrorLogViewTestMixin,
    GlobalErrorLogViewTestMixin
)


class GlobalErrorLogViewTestCase(
    ErrorLogPartitionEntryTestMixin, GlobalErrorLogViewTestMixin,
    GenericViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_error_log_object()
        self._create_test_error_log_entry()

    def test_global_error_log_partition_entry_list_view_no_permission(self):
        self._clear_events()

        response = self._request_global_error_log_partition_entry_list_view()
        self.assertNotContains(
            response=response, text=self._test_error_log_entry.text,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_global_error_log_partition_entry_list_view_with_access(self):
        self.grant_access(
            obj=self._test_object, permission=permission_error_log_entry_view
        )

        self._clear_events()

        response = self._request_global_error_log_partition_entry_list_view()
        self.assertContains(
            response=response, text=self._test_error_log_entry.text,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class ErrorLoggingViewTestCase(
    ErrorLogPartitionEntryTestMixin, ErrorLogViewTestMixin,
    GenericViewTestCase
):
    def setUp(self):
        super().setUp()
        self._create_test_error_log_object()
        self._create_test_error_log_entry()

    def test_object_error_log_view_no_permission(self):
        self._clear_events()

        response = self._request_object_error_log_list_view()
        self.assertNotContains(
            response=response, text=self._test_error_log_entry.text,
            status_code=404
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_object_error_log_entry_list_view_with_access(self):
        self.grant_access(
            obj=self._test_object, permission=permission_error_log_entry_view
        )

        self._clear_events()

        response = self._request_object_error_log_list_view()
        self.assertContains(
            response=response, text=self._test_error_log_entry.text,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_object_error_log_clear_view_no_permission(self):
        self._clear_events()

        response = self._request_object_error_log_clear_view()
        self.assertEqual(response.status_code, 404)

        self.assertNotEqual(self._test_object.error_log.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_object_error_log_clear_view_with_access(self):
        self.grant_access(
            obj=self._test_object,
            permission=permission_error_log_entry_delete
        )

        self._clear_events()

        response = self._request_object_error_log_clear_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self._test_object.error_log.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_object)
        self.assertEqual(events[0].verb, event_error_log_deleted.id)

    def test_object_error_log_entry_delete_view_no_permission(self):
        test_object_error_log_entry_count = self._test_object.error_log.count()

        self._clear_events()

        response = self._request_object_error_log_entry_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_object.error_log.count(),
            test_object_error_log_entry_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_object_error_log_entry_delete_view_with_access(self):
        self.grant_access(
            obj=self._test_object,
            permission=permission_error_log_entry_delete
        )

        test_object_error_log_entry_count = self._test_object.error_log.count()

        self._clear_events()

        response = self._request_object_error_log_entry_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_object.error_log.count(),
            test_object_error_log_entry_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_object)
        self.assertEqual(events[0].verb, event_error_log_deleted.id)



from mayan.apps.common.tests.mixins import ObjectCopyTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from .mixins.workflow_template_state_action_mixins import WorkflowTemplateStateActionTestMixin


class WorkflowTemplateCopyTestCase(
    ObjectCopyTestMixin, WorkflowTemplateStateActionTestMixin, BaseTestCase
):
    auto_create_test_workflow_template = False
    auto_create_test_workflow_template_state = False
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_workflow_template(add_test_document_type=True)
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_state()
        self._create_test_workflow_template_transition()
        self._create_test_workflow_template_state_action()
        self._test_object = self._test_workflow_template



from rest_framework import status

from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import (
    event_signature_capture_created, event_signature_capture_deleted,
    event_signature_capture_edited
)
from ..permissions import (
    permission_signature_capture_create, permission_signature_capture_delete,
    permission_signature_capture_edit, permission_signature_capture_view
)

from .mixins import SignatureCaptureAPIViewTestMixin


class SignatureCaptureAPIViewTestCase(
    DocumentTestMixin, SignatureCaptureAPIViewTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_document_stub()

    def test_signature_capture_create_api_view_no_permission(self):
        signature_capture_count = self._test_document.signature_captures.count()

        self._clear_events()

        response = self._request_test_signature_capture_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_create
        )

        signature_capture_count = self._test_document.signature_captures.count()

        self._clear_events()

        response = self._request_test_signature_capture_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_signature_capture)
        self.assertEqual(events[0].verb, event_signature_capture_created.id)

    def test_trashed_document_signature_capture_create_api_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_create
        )

        signature_capture_count = self._test_document.signature_captures.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_delete_api_view_no_permission(self):
        self._create_test_signature_capture()

        signature_capture_count = self._test_document.signature_captures.count()

        self._clear_events()

        response = self._request_test_signature_capture_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_delete_api_view_with_document_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_delete
        )

        signature_capture_count = self._test_document.signature_captures.count()

        self._clear_events()

        response = self._request_test_signature_capture_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_signature_capture_deleted.id)

    def test_signature_capture_delete_api_view_with_signature_capture_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_delete
        )

        signature_capture_count = self._test_document.signature_captures.count()

        self._clear_events()

        response = self._request_test_signature_capture_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_signature_capture_deleted.id)

    def test_trashed_document_signature_capture_delete_api_view_with_document_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_delete
        )

        signature_capture_count = self._test_document.signature_captures.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_capture_delete_api_view_with_signature_capture_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_delete
        )

        signature_capture_count = self._test_document.signature_captures.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self._test_document.signature_captures.count(),
            signature_capture_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_detail_api_view_no_permission(self):
        self._create_test_signature_capture()

        self._clear_events()

        response = self._request_test_signature_capture_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_detail_api_view_with_document_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        response = self._request_test_signature_capture_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['text'], self._test_signature_capture.text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_detail_api_view_with_signature_capture_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        response = self._request_test_signature_capture_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['text'], self._test_signature_capture.text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_capture_detail_api_view_with_document_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_capture_detail_api_view_with_signature_capture_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_edit_api_view_via_patch_no_permission(self):
        self._create_test_signature_capture()

        signature_capture_text = self._test_signature_capture.text

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_signature_capture.refresh_from_db()
        self.assertEqual(self._test_signature_capture.text, signature_capture_text)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_edit_api_view_via_patch_with_document_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_signature_capture.refresh_from_db()
        self.assertNotEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_signature_capture)
        self.assertEqual(events[0].verb, event_signature_capture_edited.id)

    def test_signature_capture_edit_api_view_via_patch_with_signature_capture_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_signature_capture.refresh_from_db()
        self.assertNotEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_signature_capture)
        self.assertEqual(events[0].verb, event_signature_capture_edited.id)

    def test_trashed_document_signature_capture_edit_api_view_via_patch_with_document_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_signature_capture.refresh_from_db()
        self.assertEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_capture_edit_api_view_via_patch_with_signature_capture_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_signature_capture.refresh_from_db()
        self.assertEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_edit_api_view_via_put_no_permission(self):
        self._create_test_signature_capture()

        signature_capture_text = self._test_signature_capture.text

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='put')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_signature_capture.refresh_from_db()
        self.assertEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_edit_api_view_via_put_with_document_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='put')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_signature_capture.refresh_from_db()
        self.assertNotEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_signature_capture)
        self.assertEqual(events[0].verb, event_signature_capture_edited.id)

    def test_signature_capture_edit_api_view_via_put_with_signature_capture_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='put')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_signature_capture.refresh_from_db()
        self.assertNotEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_signature_capture)
        self.assertEqual(events[0].verb, event_signature_capture_edited.id)

    def test_trashed_document_signature_capture_edit_api_view_via_put_with_document_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='put')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_signature_capture.refresh_from_db()
        self.assertEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_capture_edit_api_view_via_put_with_signature_capture_access(self):
        self._create_test_signature_capture()

        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_edit
        )

        signature_capture_text = self._test_signature_capture.text

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_edit_api_view(verb='put')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_signature_capture.refresh_from_db()
        self.assertEqual(
            self._test_signature_capture.text, signature_capture_text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_list_api_view_no_permission(self):
        self._create_test_signature_capture()

        self._clear_events()

        response = self._request_test_signature_capture_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_list_api_view_with_document_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        response = self._request_test_signature_capture_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['text'],
            self._test_signature_capture.text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_signature_capture_list_api_view_with_signature_capture_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._clear_events()

        response = self._request_test_signature_capture_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['text'],
            self._test_signature_capture.text
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_capture_list_api_view_with_document_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_document,
            permission=permission_signature_capture_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_signature_capture_list_api_view_with_signature_capture_access(self):
        self._create_test_signature_capture()
        self.grant_access(
            obj=self._test_signature_capture,
            permission=permission_signature_capture_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_signature_capture_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)
